@@ -1,8 +1,8 @@
 import os
 import shutil
 from datetime import datetime
-from pathlib import Path
 from functools import partial
+from pathlib import Path
 from typing import Optional, List
 
 import click
@@ -11,13 +11,13 @@ from rich.prompt import Confirm
 from yaspin import yaspin
 
 from grid.cli import rich_click
-from grid.cli.exceptions import GridError
 from grid.cli.observables import BaseObservable
 from grid.cli.rich_click import deprecate_option
 from grid.sdk import env
 from grid.sdk.client import create_swagger_client
 from grid.sdk.datastores import Datastore, list_datastores
 from grid.sdk.rest import GridRestClient
+from grid.sdk.user_messaging import info, questions
 from grid.sdk.utils.datastore_uploads import (
     resume_datastore_upload,
     find_incomplete_datastore_upload,
@@ -28,7 +28,7 @@ from grid.sdk.utils.datastore_uploads import (
 WARNING_STR = click.style('WARNING', fg='yellow')
 
 
-@click.group(invoke_without_command=True)
+@rich_click.group(invoke_without_command=True)
 @click.pass_context
 @click.option(
     '--global',
@@ -101,16 +101,14 @@ def datastore(ctx, cluster_id: str, is_global: bool, show_incomplete: bool) -> N
                     size = ds.size
                     owner = ds.user.username
                     team = ds.user.team_name
-                    status = ds.snapshot_status
-                    status = status.replace("DATASTORE_OPTIMIZATION_STATUS_", "").lower()
+                    status = ds.status
                     table_rows.append([ds.name, ds.cluster_id, str(ds.version), size, created, owner, team, status])
             else:
                 table_cols = ["Name", "Cluster ID", "Version", "Size", "Created At", "Status"]
                 for ds in sorted(datastores, key=lambda k: (k.name, k.version)):
                     created = f'{ds.created_at:%Y-%m-%d %H:%M}'
                     size = ds.size
-                    status = ds.snapshot_status
-                    status = status.replace("DATASTORE_OPTIMIZATION_STATUS_", "").lower()
+                    status = ds.status
                     table_rows.append([ds.name, ds.cluster_id, str(ds.version), size, created, status])
 
         table = BaseObservable.create_table(columns=table_cols)
@@ -145,13 +143,9 @@ def _resume():
                 modified_files = ds_work.check_for_modified_files()
                 status.stop()
                 if len(modified_files) > 0:
-                    files_changed_dialogue = '\n'.join([f"- {f.absolute_path}" for f in modified_files])
                     should_resume = Confirm.ask(
-                        "Files have been modified on disk since the last datastore upload began:\n\n"
-                        f"{files_changed_dialogue} \n\n"
-                        "[b]Are you sure that you would like to continue uploading the original files which remain?[/b] \n"
-                        "[i]NOTE: Any files which have already been uploaded will not be updated with new content[/i]",
-                        default=False,
+                        questions.datastore_ask_if_should_resume_after_files_modified(modified_files),
+                        default=True,
                     )
                     if not should_resume:
                         click.echo("Exiting!")
@@ -254,9 +248,61 @@ def delete(ctx, name: str, version: int, cluster: str) -> None:
     required=False,
     help='cluster id to create the datastore on. (Bring Your Own Cloud Customers Only).'
 )
+@click.option(
+    "--hpd",
+    "_fsx_enabled",
+    is_flag=True,
+    default=False,
+    help=(
+        "(beta) Use this flag to provision a HPD datastore backed by AWS FSx for Lustre. This "
+        "type of datastore is automatically updated whenever new files are added or deleted from the source "
+        "S3 bucket. Please take this into account when creating workflows around such Datastores. \n\n"
+        "This feature is only available to BYOC customers. "
+    ),
+)
+@click.option(
+    "--hpd-throughput",
+    "_fsx_throughput_alias",
+    type=str,
+    default="low",
+    help=(
+        "(beta) Throughput setting for HPDs. Select one of [low, medium, high]. \n\n"
+        "low (default): 125mb/s of throughput per tib of Datastore capacity. Recommended for Datastores "
+        "that will be used by one or two experiments or sessions simultaneously. \n\n"
+        "medium: 500mb/s per tib of Datastore capacity. Recommended for Datastores that will be used by "
+        "multiple experiments or sessions simultaneously. \n\n"
+        "high: highest possible throughput of 1000mb/s per tib of Datastore capacity. Recommended when "
+        "maximum performance is necessary to run a very high number of experiments. \n\n"
+        "Please note that for a single session or experiment, selecting the medium or "
+        "high settings will yield diminishing returns. In these cases, we recommended "
+        "selecting the low (default) throughput option. \n\n"
+        "Because HPDs offer elevated performance, please note that they can incur higher monthly "
+        "costs than regular Grid Datastores, especially for the medium or high throughput options. \n\n"
+        "This feature is only available to BYOC customers. "
+    ),
+)
+@click.option(
+    "--hpd-capacity",
+    "_fsx_capacity_gib",
+    type=int,
+    default=1200,
+    help=(
+        "(beta) Capacity setting for HPDs in GiB. Must be 1200, 2400 or "
+        "a multiple of 2400, up to 64800. \n\n"
+        "This feature is only available to BYOC customers. "
+    ),
+)
 @click.pass_context
 def create(
-    ctx, source: str, source_: str, cluster: str, _no_copy_data_source: bool, name: Optional[str] = None
+    ctx,
+    source: str,
+    source_: str,
+    cluster: str,
+    _no_copy_data_source: bool,
+    _fsx_enabled: bool,
+    _fsx_throughput_alias: str,
+    _fsx_capacity_gib: int,
+    name: Optional[str] = None
 ) -> None:
     """Creates a datastore from SOURCE.
 
@@ -288,6 +334,7 @@ def create(
 
         grid datastore create https://cs.nyu.edu/~roweis/data/EachMovieData.tar.gz
     """
+
     if source_ is not None:
         source = source_
     try:
@@ -295,24 +342,46 @@ def create(
         incomplete_id = find_incomplete_datastore_upload(Path(env.GRID_DIR))
         if incomplete_id is not None:
             should_resume = Confirm.ask(
-                "[b]A previous datastore upload was interrupted before it was able to complete.[/b]\n\n"
-                "Do you wish to resume this upload [bold green](yes/y)[reset]? If not [bold red](no/n)[reset], \n"
-                "then the [i]progress on the interrupted upload will be deleted[/i], \n"
-                "and you will have to upload the dataset in full.\n",
+                questions.datastore_ask_if_should_resume_upload(source),
                 default=True,
             )
             if should_resume is True:
                 _resume()
             else:
                 remove_datastore_work_state(grid_dir=Path(env.GRID_DIR), datastore_id=incomplete_id)
-                dstore = Datastore(name=name, source=source, cluster_id=cluster, s3_no_copy=_no_copy_data_source)
-                click.echo(f"\ncreating datastore from {source}")
+                dstore = Datastore(
+                    name=name,
+                    source=source,
+                    cluster_id=cluster,
+                    s3_no_copy=_no_copy_data_source,
+                    fsx_enabled=_fsx_enabled,
+                    fsx_throughput_alias=_fsx_throughput_alias,
+                    fsx_capacity_gib=_fsx_capacity_gib
+                )
+                click.echo(
+                    f"\n{info.datastore_currently_creating_datastore(name=dstore.name, version=str(dstore.version), cluster=cluster)}"
+                )
                 dstore.upload()
         else:
             with Console().status("[bold yellow]Indexing datastore uploads....", spinner_style="yellow") as status:
-                dstore = Datastore(name=name, source=source, cluster_id=cluster, s3_no_copy=_no_copy_data_source)
-                click.echo(f"\ncreating datastore from {source}")
+                dstore = Datastore(
+                    name=name,
+                    source=source,
+                    cluster_id=cluster,
+                    s3_no_copy=_no_copy_data_source,
+                    fsx_enabled=_fsx_enabled,
+                    fsx_throughput_alias=_fsx_throughput_alias,
+                    fsx_capacity_gib=_fsx_capacity_gib
+                )
+                click.echo(
+                    f"\n{info.datastore_currently_creating_datastore(name=dstore.name, version=str(dstore.version), cluster=cluster)}"
+                )
                 status.stop()
                 dstore.upload()
     except Exception as e:
         raise click.ClickException(e)
+
+    click.echo(info.datastore_uploaded_successfully(name=name, version=str(dstore.version), cluster=cluster))
+
+    if _fsx_enabled:
+        click.echo(info.datastore_fsx_total_throughput(_fsx_throughput_alias, _fsx_capacity_gib))
