@@ -1,13 +1,11 @@
 import textwrap
 import warnings
-from collections import defaultdict
-from datetime import datetime, time
+from datetime import datetime
 from pathlib import Path
 from typing import List, Optional, Union, Dict
 from urllib.parse import urlparse
 
 import click
-import rich
 
 from grid.openapi import V1GetDatastoreResponse, V1DatastoreOptimizationStatus, V1DatastoreSourceType
 from grid.openapi.rest import ApiException
@@ -25,16 +23,67 @@ from grid.sdk.rest.datastores import (
     mark_datastore_upload_complete,
 )
 from grid.sdk.user import User, user_from_logged_in_account
+from grid.sdk.user_messaging import errors
 from grid.sdk.utilities import SPECIAL_NAME_TO_SKIP_OBJECT_INIT
 from grid.sdk.utils.datastore_uploads import (
     resume_datastore_upload,
     begin_new_datastore_upload,
     find_incomplete_datastore_upload,
     load_datastore_work_state,
-    initialize_upload_work,
 )
 
-DATASTORE_TERMINAL_STATES = ['FAILED', 'SUCCEEDED']
+DATASTORE_TERMINAL_STATES = [
+    V1DatastoreOptimizationStatus.DELETED,
+    V1DatastoreOptimizationStatus.SUCCEEDED,
+    V1DatastoreOptimizationStatus.FAILED,
+]
+
+DATASTORE_FSX_THROUGHPUT_ALIASES = {
+    "low": 125,
+    "medium": 500,
+    "high": 1000,
+}
+
+
+def _datastore_optimization_status_name_to_human(phase: str) -> str:
+    """Converts datastore status enum name to cannoniacl format
+
+    Example
+    -------
+    "DATASTORE_OPTIMIZATION_STATUS_FAILED" -> "failed"
+
+    Notes
+    -----
+    See PRD for terms used: https://www.notion.so/gridai/Workload-status-observability-0b635c62db9644499d5b5fbd992385d3
+    """
+    conversion_map = {
+        V1DatastoreOptimizationStatus.DELETED:
+        "deleted",
+        V1DatastoreOptimizationStatus.FAILED:
+        "failed",
+        V1DatastoreOptimizationStatus.SUCCEEDED:
+        "available",
+        V1DatastoreOptimizationStatus.OPTIMIZING:
+        "optimizing",
+        V1DatastoreOptimizationStatus.POD_PENDING:
+        "optimizing",
+        V1DatastoreOptimizationStatus.CLOUD_RESOURCES_PENDING:
+        "optimizing",
+        # yes, this doesn't make sense... but we need to make the datastore proto
+        # spec compliant with every other resource before we can change this
+        V1DatastoreOptimizationStatus.UNSPECIFIED:
+        "uploading",
+    }
+
+    if phase not in conversion_map:
+        warnings.warn((
+            f"The operation completed, but you've found a bug! We cannot convert optimization "
+            f"status name: {phase} to a human readable format. Please report this to the grid "
+            f"support team on the community slack or at support@grid.ai."
+        ), RuntimeWarning)
+    else:
+        phase = conversion_map[phase]
+    return phase
 
 
 def fetch_datastore(datastore_name: str, datastore_version: int, cluster: str) -> 'Datastore':
@@ -94,11 +143,14 @@ class Datastore:
     _version: int
     _source: Optional[Union[str, Path]]
     _s3_no_copy: bool
-    _snapshot_status: str
+    _status: Optional[str]
     _created_at: datetime
     _user: User
     _cluster_id: str
     _size: str
+    _fsx_enabled: bool
+    _fsx_throughput_mbs_tib: int
+    _fsx_capacity_gib: int
 
     _is_deleted: bool
     _is_created: bool
@@ -112,6 +164,9 @@ class Datastore:
         user: Optional[User] = None,
         version: int = 0,
         cluster_id: Optional[str] = None,
+        fsx_enabled: bool = False,
+        fsx_throughput_alias="low",
+        fsx_capacity_gib=1200,
     ):
         """Initialize a new DataStore Object.
 
@@ -143,6 +198,12 @@ class Datastore:
             The user that owns the DataStore.
         cluster_id
             The name of the cluster that the DataStore should be uploaded to.
+        fsx_enabled
+            Whether to provision an FSx-backed datastore
+        fsx_throughput_alias
+            Which throughput tier to provision for the FSx file system. low=125mb/s/tib, medium=500mb/s/tib, high=1000mb/s/tib
+        fsx_capacity_gib
+            How much capacity to provision for the FSx file system, must be 1200, 2400 or a multiple of 2400
         """
         # --------------------------------------------------------------------------- #
         #    This should be the first block that goes into the constructor of the     #
@@ -176,6 +237,7 @@ class Datastore:
                 raise ValueError("Name is required if source is not provided.")
         else:
             try:
+
                 datastore = get_datastore_from_name(
                     client=self._client, cluster_id=cluster_id, datastore_name=name, version=version
                 )
@@ -192,14 +254,28 @@ class Datastore:
                 f"as this is auto-generated."
             )
 
+        self._fsx_throughput_mbs_tib = 0
+        self._fsx_capacity_gib = 0
+
+        if fsx_enabled is True:
+            if fsx_throughput_alias not in DATASTORE_FSX_THROUGHPUT_ALIASES:
+                raise ValueError(
+                    f"Invalid throughput {fsx_throughput_alias} provided for high performance datastore."
+                    f"Please use one of {DATASTORE_FSX_THROUGHPUT_ALIASES.keys()}."
+                )
+            else:
+                self._fsx_throughput_mbs_tib = DATASTORE_FSX_THROUGHPUT_ALIASES[fsx_throughput_alias]
+                self._fsx_capacity_gib = fsx_capacity_gib
+
         self._name = name
         self._version = 0
         self._source = source
         self._user = user
         self._cluster_id = cluster_id
         self._s3_no_copy = s3_no_copy
+        self._fsx_enabled = fsx_enabled
         self._id = None
-        self._snapshot_status = None
+        self._status = None
         self._created_at = None
         self._size = None
 
@@ -226,7 +302,8 @@ class Datastore:
         instance._version = datastore.spec.version
         instance._source = datastore.spec.source
         instance._s3_no_copy = instance._source == V1DatastoreSourceType.OBJECT_STORE_REFERENCE_ONLY
-        instance._snapshot_status = str(datastore.status.phase)  # TODO - rename to status
+        instance._fsx_enabled = instance._source == V1DatastoreSourceType.FSX
+        instance._status = datastore.status.phase
         instance._created_at = datastore.created_at
         instance._size = f"{datastore.spec.size_mib} MiB"
         instance._user = User(user_id=datastore.spec.user_id, username="", first_name="", last_name="")
@@ -305,14 +382,14 @@ class Datastore:
 
     @property
     @affirm(is_not_shallow, is_not_deleted)
-    def snapshot_status(self) -> str:
+    def status(self) -> str:
         """The status of the datastore.
         """
-        if (self._snapshot_status and
-            (self._snapshot_status.upper() not in DATASTORE_TERMINAL_STATES)) or ((self._snapshot_status is None) and
+        if (self._status and (self._status not in DATASTORE_TERMINAL_STATES)) or ((self._status is None) and
                                                                                   (self._is_created is True)):
             self._update_status()
-        return self._snapshot_status
+
+        return _datastore_optimization_status_name_to_human(self._status)
 
     @property
     @affirm(is_not_shallow, is_not_deleted)
@@ -396,8 +473,8 @@ class Datastore:
     @affirm(is_not_shallow, is_not_deleted)
     def __hash__(self):
         return hash((
-            self._name, self._id, self._version, self._size, self._created_at, self._snapshot_status, self._user,
-            self._source, self._cluster_id, self._is_deleted, self._is_created
+            self._name, self._id, self._version, self._size, self._created_at, self._status, self._user, self._source,
+            self._cluster_id, self._is_deleted, self._is_created
         ))
 
     # ---------------------  Internal Methods ----------------------
@@ -407,9 +484,9 @@ class Datastore:
         """Refreshes the``snapshot_status`` attribute by querying the Grid API.
         """
         updated = get_datastore_from_id(c=self._client, cluster_id=self._cluster_id, datastore_id=self._id)
-        self._snapshot_status = str(updated.status.phase)
+        self._status = str(updated.status.phase)
 
-    # -------------------  Public Facing Methods ----------------------
+    # ---------------------  Public Facing Methods ---------------------
 
     @affirm(is_not_shallow, is_not_deleted, is_created)
     def delete(self):
@@ -432,8 +509,12 @@ class Datastore:
             cluster_id=self._cluster_id,
             name=self._name,
             source=self._source,
-            no_copy_data_source=self._s3_no_copy
+            no_copy_data_source=self._s3_no_copy,
+            fsx_enabled=self._fsx_enabled,
+            fsx_throughput_mbs_tib=self._fsx_throughput_mbs_tib,
+            fsx_capacity_gib=self._fsx_capacity_gib
         )
+
         self._id = dstore.id
         self._version = dstore.spec.version
         self._created_at = dstore.created_at
@@ -522,8 +603,13 @@ def parse_name_from_source(source) -> str:
     try:
         parse_result = urlparse(source)
     except ValueError:
-        raise click.ClickException("Invalid source for datastore, please input only a local filepath or valid url")
+        raise click.ClickException(errors.datastore_invalid_source(source))
 
-    path = Path(parse_result.path)
-    base = path.name.split(".")[0]
-    return base.lower().strip()
+    if parse_result.path != '/' and parse_result.path != '':
+        path = Path(parse_result.path)
+        base = path.name.split(".")[0]
+        ret = base.lower().strip()
+    else:
+        ret = parse_result.netloc
+
+    return ret
